@@ -69,7 +69,7 @@ def _uniop(opname, cls=instructions.Instruction):
     return wrap
 
 
-def _uniop_intrinsic(opname):
+def _uniop_intrinsic_int(opname):
     def wrap(fn):
         @functools.wraps(fn)
         def wrapped(self, operand, name=''):
@@ -83,6 +83,20 @@ def _uniop_intrinsic(opname):
     return wrap
 
 
+def _uniop_intrinsic_float(opname):
+    def wrap(fn):
+        @functools.wraps(fn)
+        def wrapped(self, operand, name=''):
+            if not isinstance(operand.type, (types.FloatType, types.DoubleType)):
+                raise TypeError("expected a float type, got %s" % operand.type)
+            fn = self.module.declare_intrinsic(opname, [operand.type])
+            return self.call(fn, [operand], name)
+
+        return wrapped
+
+    return wrap
+
+
 def _uniop_intrinsic_with_flag(opname):
     def wrap(fn):
         @functools.wraps(fn)
@@ -109,7 +123,9 @@ def _triop_intrinsic(opname):
                         a.type,
                         b.type,
                         c.type))
-            elif not isinstance(a.type, (types.FloatType, types.DoubleType)):
+            elif not isinstance(
+                    a.type,
+                    (types.HalfType, types.FloatType, types.DoubleType)):
                 raise TypeError("expected an floating point type, got %s" % a.type)
             fn = self.module.declare_intrinsic(opname, [a.type, b.type, c.type])
             return self.call(fn, [a, b, c], name)
@@ -134,6 +150,22 @@ def _castop(opname, cls=instructions.CastInstr):
     return wrap
 
 
+def _label_suffix(label, suffix):
+    """Returns (label + suffix) or a truncated version if it's too long.
+    Parameters
+    ----------
+    label : str
+        Label name
+    suffix : str
+        Label suffix
+    """
+    if len(label) > 50:
+        nhead = 25
+        return ''.join([label[:nhead], '..', suffix])
+    else:
+        return label + suffix
+
+
 class IRBuilder(object):
     def __init__(self, block=None):
         self._block = block
@@ -245,8 +277,8 @@ class IRBuilder(object):
         for LLVM's optimizers to account for that.
         """
         bb = self.basic_block
-        bbif = self.append_basic_block(name=bb.name + '.if')
-        bbend = self.append_basic_block(name=bb.name + '.endif')
+        bbif = self.append_basic_block(name=_label_suffix(bb.name, '.if'))
+        bbend = self.append_basic_block(name=_label_suffix(bb.name, '.endif'))
         br = self.cbranch(pred, bbif, bbend)
         if likely is not None:
             br.set_weights([99, 1] if likely else [1, 99])
@@ -273,9 +305,9 @@ class IRBuilder(object):
                     # emit instructions for when the predicate is false
         """
         bb = self.basic_block
-        bbif = self.append_basic_block(name=bb.name + '.if')
-        bbelse = self.append_basic_block(name=bb.name + '.else')
-        bbend = self.append_basic_block(name=bb.name + '.endif')
+        bbif = self.append_basic_block(name=_label_suffix(bb.name, '.if'))
+        bbelse = self.append_basic_block(name=_label_suffix(bb.name, '.else'))
+        bbend = self.append_basic_block(name=_label_suffix(bb.name, '.endif'))
         br = self.cbranch(pred, bbif, bbelse)
         if likely is not None:
             br.set_weights([99, 1] if likely else [1, 99])
@@ -480,7 +512,11 @@ class IRBuilder(object):
         Bitwise integer complement:
             name = ~value
         """
-        return self.xor(value, values.Constant(value.type, -1), name=name)
+        if isinstance(value.type, types.VectorType):
+            rhs = values.Constant(value.type, (-1,) * value.type.count)
+        else:
+            rhs = values.Constant(value.type, -1)
+        return self.xor(value, rhs, name=name)
 
     def neg(self, value, name=''):
         """
@@ -707,6 +743,33 @@ class IRBuilder(object):
         self._insert(st)
         return st
 
+    def load_atomic(self, ptr, ordering, align, name=''):
+        """
+        Load value from pointer, with optional guaranteed alignment:
+            name = *ptr
+        """
+        if not isinstance(ptr.type, types.PointerType):
+            raise TypeError("cannot load from value of type %s (%r): not a pointer"
+                            % (ptr.type, str(ptr)))
+        ld = instructions.LoadAtomicInstr(self.block, ptr, ordering, align, name)
+        self._insert(ld)
+        return ld
+
+    def store_atomic(self, value, ptr, ordering, align):
+        """
+        Store value to pointer, with optional guaranteed alignment:
+            *ptr = name
+        """
+        if not isinstance(ptr.type, types.PointerType):
+            raise TypeError("cannot store to value of type %s (%r): not a pointer"
+                            % (ptr.type, str(ptr)))
+        if ptr.type.pointee != value.type:
+            raise TypeError("cannot store %s to %s: mismatching types"
+                            % (value.type, ptr.type))
+        st = instructions.StoreAtomicInstr(self.block, value, ptr, ordering, align)
+        self._insert(st)
+        return st
+
 
     #
     # Terminators APIs
@@ -820,6 +883,39 @@ class IRBuilder(object):
         self._insert(instr)
         return instr
 
+    # Vector Operations APIs
+
+    def extract_element(self, vector, idx, name=''):
+        """
+        Returns the value at position idx.
+        """
+        instr = instructions.ExtractElement(self.block, vector, idx, name=name)
+        self._insert(instr)
+        return instr
+
+    def insert_element(self, vector, value, idx, name=''):
+        """
+        Returns vector with vector[idx] replaced by value.
+        The result is undefined if the idx is larger or equal the vector length.
+        """
+        instr = instructions.InsertElement(self.block, vector, value, idx,
+                                           name=name)
+        self._insert(instr)
+        return instr
+
+    def shuffle_vector(self, vector1, vector2, mask, name=''):
+        """
+        Constructs a permutation of elements from *vector1* and *vector2*.
+        Returns a new vector in the same length of *mask*.
+
+        * *vector1* and *vector2* must have the same element type.
+        * *mask* must be a constant vector of integer types.
+        """
+        instr = instructions.ShuffleVector(self.block, vector1, vector2, mask,
+                                           name=name)
+        self._insert(instr)
+        return instr
+
     # Aggregate APIs
 
     def extract_value(self, agg, idx, name=''):
@@ -901,19 +997,19 @@ class IRBuilder(object):
         self._insert(inst)
         return inst
 
-    @_uniop_intrinsic("llvm.bswap")
+    @_uniop_intrinsic_int("llvm.bswap")
     def bswap(self, cond):
         """
         Used to byte swap integer values with an even number of bytes (positive multiple of 16 bits)
         """
 
-    @_uniop_intrinsic("llvm.bitreverse")
+    @_uniop_intrinsic_int("llvm.bitreverse")
     def bitreverse(self, cond):
         """
         Reverse the bitpattern of an integer value; for example 0b10110110 becomes 0b01101101.
         """
 
-    @_uniop_intrinsic("llvm.ctpop")
+    @_uniop_intrinsic_int("llvm.ctpop")
     def ctpop(self, cond):
         """
         Counts the number of bits set in a value.
@@ -922,13 +1018,15 @@ class IRBuilder(object):
     @_uniop_intrinsic_with_flag("llvm.ctlz")
     def ctlz(self, cond, flag):
         """
-        Counts the number of leading zeros in a variable.
+        Counts leading zero bits in *value*. Boolean *flag* indicates whether
+        the result is defined for ``0``.
         """
 
     @_uniop_intrinsic_with_flag("llvm.cttz")
     def cttz(self, cond, flag):
         """
-        Counts the number of trailing zeros in a variable.
+        Counts trailing zero bits in *value*. Boolean *flag* indicates whether
+        the result is defined for ``0``.
         """
 
     @_triop_intrinsic("llvm.fma")
@@ -936,3 +1034,24 @@ class IRBuilder(object):
         """
         Perform the fused multiply-add operation.
         """
+
+    def convert_from_fp16(self, a, to=None, name=''):
+        """
+        Convert from an i16 to the given FP type
+        """
+        if not to:
+            raise TypeError("expected a float return type")
+        if not isinstance(to, (types.FloatType, types.DoubleType)):
+            raise TypeError("expected a float type, got %s" % to)
+        if a.type != types.IntType(16):
+            raise TypeError("expected an i16 type, got %s" % a.type)
+
+        opname = 'llvm.convert.from.fp16'
+        fn = self.module.declare_intrinsic(opname, [to])
+        return self.call(fn, [a], name)
+
+    @_uniop_intrinsic_float("llvm.convert.to.fp16")
+    def convert_to_fp16(self, a):
+        """
+        Convert the given FP number to an i16
+        """
