@@ -6,6 +6,7 @@ Instructions are in the instructions module.
 from __future__ import print_function, absolute_import
 
 import string
+import re
 
 from .. import six
 from . import types, _utils
@@ -16,6 +17,8 @@ _VALID_CHARS = (frozenset(map(ord, string.ascii_letters)) |
                 frozenset(map(ord, string.digits)) |
                 frozenset(map(ord, ' !#$%&\'()*+,-./:;<=>?@[]^_`{|}~')))
 
+_SIMPLE_IDENTIFIER_RE = re.compile(r"[-a-zA-Z$._][-a-zA-Z$._0-9]*$")
+
 
 def _escape_string(text, _map={}):
     """
@@ -103,6 +106,12 @@ class _Undefined(object):
     """
     'undef': a value for undefined values.
     """
+    def __new__(cls):
+        try:
+            return Undefined
+        except NameError:
+            return object.__new__(_Undefined)
+
 
 Undefined = _Undefined()
 
@@ -116,9 +125,7 @@ class Constant(_StrCaching, _StringReferenceCaching, _ConstOpMixin, Value):
         assert isinstance(typ, types.Type)
         assert not isinstance(typ, types.VoidType)
         self.type = typ
-        if isinstance(constant, (list, tuple)):
-            # Recursively wrap aggregate constants
-            constant = typ.wrap_constant_value(constant)
+        constant = typ.wrap_constant_value(constant)
         self.constant = constant
 
     def _to_string(self):
@@ -546,9 +553,10 @@ class FunctionAttributes(AttributeSet):
         'sanitize_memory', 'sanitize_thread', 'ssp',
         'sspreg', 'sspstrong', 'uwtable'])
 
-    def __init__(self):
+    def __init__(self, args=()):
         self._alignstack = 0
         self._personality = None
+        super(FunctionAttributes, self).__init__(args)
 
     @property
     def alignstack(self):
@@ -672,10 +680,11 @@ class ArgumentAttributes(AttributeSet):
                         'nocapture', 'nonnull', 'returned', 'signext',
                         'sret', 'zeroext'])
 
-    def __init__(self):
+    def __init__(self, args=()):
         self._align = 0
         self._dereferenceable = 0
         self._dereferenceable_or_null = 0
+        super(ArgumentAttributes, self).__init__(args)
 
     @property
     def align(self):
@@ -784,7 +793,7 @@ class Block(NamedValue):
         return self.parent.module
 
     def descr(self, buf):
-        buf.append("{0}:\n".format(self.name))
+        buf.append("{0}:\n".format(self._format_name()))
         buf += ["  {0}\n".format(instr) for instr in self.instructions]
 
     def replace(self, old, new):
@@ -798,6 +807,17 @@ class Block(NamedValue):
         for bb in self.parent.basic_blocks:
             for instr in bb.instructions:
                 instr.replace_usage(old, new)
+    
+    def _format_name(self):
+        # Per the LLVM Language Ref on identifiers, names matching the following 
+        # regex do not need to be quoted: [%@][-a-zA-Z$._][-a-zA-Z$._0-9]*
+        # Otherwise, the identifier must be quoted and escaped.
+        name = self.name
+        if not _SIMPLE_IDENTIFIER_RE.match(name):
+            name = name.replace('\\', '\\5c').replace('"', '\\22')
+            name = '"{0}"'.format(name)
+        return name
+
 
 
 class BlockAddress(Value):
